@@ -16,7 +16,7 @@
 #include "drawlayer.h"
 #include "drawsurface.h"
 #include "drawpath3.h"
-#include "win32helpers.h"
+#include "seconds.h"
 
 #if defined(_WIN32)
 #include <Windows.h>
@@ -525,15 +525,7 @@ bool picture::texprocess(const string& texname, const string& outname,
           string dvipsrc=getSetting<string>("dir");
           if(dvipsrc.empty()) dvipsrc=systemDir;
           dvipsrc += dirsep+"nopapersize.ps";
-#if !defined(_WIN32)
-          setenv("DVIPSRC",dvipsrc.c_str(),1);
-#else
-          auto setEnvResult = SetEnvironmentVariableA("DVIPSRC",dvipsrc.c_str());
-          if (!setEnvResult)
-          {
-              camp::reportError("Cannot set DVIPSRC environment variable");
-          }
-#endif
+          setenv("DVIPSRC",dvipsrc.c_str(),true);
           string papertype=getSetting<string>("papertype") == "letter" ?
             "letterSize" : "a4size";
           cmd.push_back(getSetting<string>("dvips"));
@@ -854,7 +846,7 @@ bool picture::postprocess(const string& prename, const string& outname,
   if(pdftex || !epsformat) {
     if(pdfformat) {
       if(pdftex) {
-        status=rename(prename.c_str(),outname.c_str());
+        status=renameOverwrite(prename.c_str(),outname.c_str());
         if(status != 0)
           reportError("Cannot rename "+prename+" to "+outname);
       } else status=epstopdf(prename,outname);
@@ -920,7 +912,7 @@ bool picture::postprocess(const string& prename, const string& outname,
 bool picture::display(const string& outname, const string& outputformat,
                       bool wait, bool view, bool epsformat)
 {
-  static mem::map<CONST string,int> pids;
+  static mem::map<const string,int> pids;
   if (settings::view() && view)
   {
     int status;
@@ -930,7 +922,7 @@ bool picture::display(const string& outname, const string& outputformat,
 
     if(epsformat || pdfformat) {
       // Check to see if there is an existing viewer for this outname.
-      mem::map<CONST string,int>::iterator const p=pids.find(outname);
+      mem::map<const string,int>::iterator const p=pids.find(outname);
       bool running=(p != pids.end());
       string Viewer=
         pdfformat ?
@@ -1564,6 +1556,7 @@ bool picture::shipout3(const string& prefix, const string& format,
         com.view=View;
         if(Wait)
           pthread_mutex_lock(&readyLock);
+        allowRender=true;
         wait(initSignal,initLock);
         endwait(initSignal,initLock);
         static bool initialize=true;
