@@ -18,8 +18,6 @@
 #include <Windows.h>
 #include <io.h>
 #define isatty _isatty
-
-#include "win32helpers.h"
 #else
 #include <unistd.h>
 #endif
@@ -150,151 +148,116 @@ const string dirsep="\\";
 /**
  * Use key to look up an entry in the MSWindows registry,
  * @param baseRegLocation base location for a key
- * @param key Key to look up, respecting wild cards
+ * @param key Key to look up, respecting wild cards. Note that wildcards
+ * only support single-level glob. Recursive globs are not supported.
+ * @param value Value to look up
  * @remark Wildcards can only be in keys, not in the final value
- * @return Entry value, or an empty string if not found
+ * @return Entry value, or nullopt if not found
  */
-string getEntry(const HKEY& baseRegLocation, const string& key)
+optional<string>
+getEntry(HKEY const& baseRegLocation, string const& key, string const& value)
 {
-  string path=key;
-  size_t star;
-  string head;
-  while((star=path.find('*')) < string::npos)
-  {
-    // has asterisk in the path
-
-    string prefix=path.substr(0,star);
-    string pathSuffix=path.substr(star+1);
-    // path = prefix [*] pathSuffix
-    size_t slash=pathSuffix.find('\\');
-    if(slash < string::npos) {
-      // pathsuffix is not leaf yet
-      // pathSuffix = <new path suffix>[/<new path>]
-      path=pathSuffix.substr(slash);
-      pathSuffix=pathSuffix.substr(0,slash);
-    } else {
-      // pathSuffix is entirely part of registry value name
-      // pathSuffix = <new path>, new path suffix is empty
-      path=pathSuffix;
-      pathSuffix="";
-    }
-    string directory=head+stripFile(prefix);
-    string file=stripDir(prefix);
-    // prefix = directory/file
-    // [old path] = directory/file [*] pathSuffix [/path]
-    // or, if asterisk is in the leaf
-    // [old path] = directory/file[*]path, pathSuffix is empty
-
-    camp::w32::RegKeyWrapper currRegDirectory;
-    if (RegOpenKeyExA(baseRegLocation, directory.c_str(), 0, KEY_READ, currRegDirectory.put()) != ERROR_SUCCESS)
-    {
-      currRegDirectory.release();
-      return "";
-    }
-
-    DWORD numSubKeys;
-    DWORD longestSubkeySize;
-
-    if (RegQueryInfoKeyA(
-                currRegDirectory.getKey(),
-                nullptr, nullptr, nullptr,
-                &numSubKeys, &longestSubkeySize,
-                nullptr, nullptr, nullptr, nullptr, nullptr, nullptr) != ERROR_SUCCESS)
-    {
-      numSubKeys = 0;
-      longestSubkeySize = 0;
-    }
-    mem::vector<char> subkeyBuffer(longestSubkeySize + 1);
-    bool found=false;
-
-    string rsuffix= pathSuffix;
-    reverse(rsuffix.begin(), rsuffix.end());
-    for (DWORD i=0;i<numSubKeys;++i)
-    {
-      DWORD cchValue=longestSubkeySize + 1;
-
-      if (auto const regQueryResult= RegEnumKeyExA(
-                  currRegDirectory.getKey(),
-                  i,
-                  subkeyBuffer.data(),
-                  &cchValue,
-                  nullptr,
-                  nullptr,
-                  nullptr,
-                  nullptr
-          );
-          regQueryResult != ERROR_SUCCESS)
-      {
-        continue;
-      }
-
-      string const dname(subkeyBuffer.data());
-      string rdname=dname;
-      reverse(rdname.begin(),rdname.end());
-      if(dname.substr(0,file.size()) == file &&
-         rdname.substr(0, pathSuffix.size()) == rsuffix) {
-        // dname matches file [*} pathSuffix
-        head=directory+dname;
-        found=true;
-        break;
-      }
-    }
-
-    if (!found)
-    {
-      return "";
-    }
+  string path= key;
+  if (key.find('\\') == 0) {
+    path= path.substr(1);// strip the prefix separator
   }
 
-  if (path.find('\\') == 0)
-  {
-    path = path.substr(1); // strip the prefix separator
+  size_t const star= path.find('*');
+  if (star == string::npos) {
+    // absolute path, can return right away
+    DWORD dataSize= 0;
+    if (RegGetValueA(
+                baseRegLocation, path.c_str(), value.c_str(), RRF_RT_REG_SZ,
+                nullptr, nullptr, &dataSize
+        ) != ERROR_SUCCESS) {
+      return nullopt;
+    }
+
+    mem::vector<BYTE> outputBuffer(dataSize);
+
+    if (RegGetValueA(
+                baseRegLocation, path.c_str(), value.c_str(), RRF_RT_REG_SZ,
+                nullptr, outputBuffer.data(), &dataSize
+        ) != ERROR_SUCCESS) {
+      return nullopt;
+    }
+
+    return make_optional<string>(
+            reinterpret_cast<char const*>(outputBuffer.data())
+    );
   }
 
-  if (path == "@")
-  {
-    path= "";// default registry key
+  // has a glob, search until we find one
+  string const prefix= path.substr(0, star);
+  string const pathSuffix= path.substr(star + 1);
+
+  // open the key in prefix
+
+  camp::w32::RegKeyWrapper directoryWithPrefix;
+  if (RegOpenKeyExA(
+              baseRegLocation, prefix.c_str(), 0, KEY_READ,
+              directoryWithPrefix.put()
+      ) != ERROR_SUCCESS) {
+    return nullopt;// prefix path does not exist, or some other error
   }
 
-  DWORD requiredStrSize=0;
-  // FIXME: Add handling of additional types
-  if (RegGetValueA(baseRegLocation, head.c_str(), path.c_str(), RRF_RT_REG_SZ, nullptr, nullptr, &requiredStrSize) !=
-    ERROR_SUCCESS)
-  {
-    return "";
+  DWORD numSubKeys= 0;
+  DWORD longestSubkeySize= 0;
+
+  // querying # of subkeys + their longest path length
+  if (RegQueryInfoKeyA(
+              directoryWithPrefix.getKey(), nullptr, nullptr, nullptr,
+              &numSubKeys, &longestSubkeySize, nullptr, nullptr, nullptr,
+              nullptr, nullptr, nullptr
+      ) != ERROR_SUCCESS) {
+    return nullopt;
   }
 
-  mem::vector<BYTE> outputBuffer(requiredStrSize);
+  mem::vector<CHAR> subkeyBuffer(longestSubkeySize + 1);
 
-  if (auto const retCheck = RegGetValueA(baseRegLocation, head.c_str(), path.c_str(),
-              RRF_RT_REG_SZ,
-              nullptr,
-              outputBuffer.data(),
-              &requiredStrSize
-      );
-      retCheck != ERROR_SUCCESS)
-  {
-    return "";
-  }
+  for (DWORD i= 0; i < numSubKeys; ++i) {
+    DWORD cchValue= longestSubkeySize + 1;
 
-  return reinterpret_cast<char const*>(outputBuffer.data());
-}
-
-// Use key to look up an entry in the MSWindows registry, respecting wild cards
-string getEntry(const string& key)
-{
-  for (HKEY const keyToSearch : { HKEY_LOCAL_MACHINE, HKEY_CURRENT_USER })
-  {
-    camp::w32::RegKeyWrapper baseRegKey;
-    if (RegOpenKeyExA(keyToSearch, "Software", 0, KEY_READ, baseRegKey.put()) != ERROR_SUCCESS)
-    {
-      baseRegKey.release();
+    // get subkey's name
+    if (RegEnumKeyExA(
+                directoryWithPrefix.getKey(), i, subkeyBuffer.data(), &cchValue,
+                nullptr, nullptr, nullptr, nullptr
+        ) != ERROR_SUCCESS) {
       continue;
     }
 
-    if (string entry=getEntry(baseRegKey.getKey(),key); !entry.empty())
-    {
-      return entry;
+    // open the subkey
+    camp::w32::RegKeyWrapper searchKey;
+    if (RegOpenKeyExA(
+                directoryWithPrefix.getKey(), subkeyBuffer.data(), 0, KEY_READ,
+                searchKey.put()
+        ) != ERROR_SUCCESS) {
+      continue;
+    }
+
+    // do a recursive search starting at the opened key
+    if (auto retResult= getEntry(searchKey.getKey(), pathSuffix, value);
+        retResult.has_value()) {
+      return retResult;
+    }
+  }
+  return nullopt;
+}
+
+// Use key to look up an entry in the MSWindows registry, respecting wild cards
+string getEntry(const string& key, const string& value)
+{
+  for (HKEY const keyToSearch : {HKEY_LOCAL_MACHINE, HKEY_CURRENT_USER}) {
+    camp::w32::RegKeyWrapper baseRegKey;
+    if (RegOpenKeyExA(keyToSearch, "SOFTWARE", 0, KEY_READ, baseRegKey.put()) !=
+        ERROR_SUCCESS) {
+      baseRegKey.release();
+      continue;
+    }
+    optional<string> entry= getEntry(baseRegKey.getKey(), key, value);
+
+    if (entry.has_value()) {
+      return entry.value();
     }
   }
 
@@ -303,21 +266,25 @@ string getEntry(const string& key)
 
 void queryRegistry()
 {
-  defaultGhostscriptLibrary= getEntry(R"(GPL Ghostscript\*\GS_DLL)");
-  if(defaultGhostscriptLibrary.empty())
-    defaultGhostscriptLibrary=getEntry(R"(AFPL Ghostscript\*\GS_DLL)");
+  defaultGhostscriptLibrary= getEntry(R"(GPL Ghostscript\*)", "GS_DLL");
+  if (defaultGhostscriptLibrary.empty())
+    defaultGhostscriptLibrary= getEntry(R"(AFPL Ghostscript\*)", "GS_DLL");
 
-  string gslib=stripDir(defaultGhostscriptLibrary);
-  defaultGhostscript=stripFile(defaultGhostscriptLibrary)+
-    ((gslib.empty() || gslib.substr(5,2) == "32") ? "gswin32c.exe" : "gswin64c.exe");
+  string gslib= stripDir(defaultGhostscriptLibrary);
+  defaultGhostscript=
+          stripFile(defaultGhostscriptLibrary) +
+          ((gslib.empty() || gslib.substr(5, 2) == "32") ? "gswin32c.exe"
+                                                         : "gswin64c.exe");
 
-  if (string const s= getEntry(R"(Microsoft\Windows\CurrentVersion\App Paths\Asymptote\Path)"); !s.empty())
-  {
+  string const s= getEntry(
+          R"(Microsoft\Windows\CurrentVersion\App Paths\Asymptote)", "Path"
+  );
+  if (!s.empty()) {
     docdir= s;
   }
   // An empty systemDir indicates a TeXLive build
-  if(!systemDir.empty() && !docdir.empty())
-    systemDir=docdir;
+  if (!systemDir.empty() && !docdir.empty())
+    systemDir= docdir;
 }
 
 #endif
@@ -400,10 +367,10 @@ bool warn(const string& s)
 
 // The dictionaries of long options and short options.
 struct option;
-typedef mem::map<CONST string, option *> optionsMap_t;
+typedef mem::map<const string, option *> optionsMap_t;
 optionsMap_t optionsMap;
 
-typedef mem::map<CONST char, option *> codeMap_t;
+typedef mem::map<const char, option *> codeMap_t;
 codeMap_t codeMap;
 
 struct option : public gc {
