@@ -14,7 +14,7 @@
 #include "errormsg.h"
 #include "util.h"
 #include "runtime.h"
-#include "process.h"
+#include "asyprocess.h"
 
 #include "profiler.h"
 
@@ -159,10 +159,10 @@ void stack::marshall(size_t args, stack::vars_t vars)
 #ifdef PROFILE
 
 #ifndef DEBUG_FRAME
-#warning "profiler needs DEBUG_FRAME for function names"
+#pragma message("WARNING: profiler needs DEBUG_FRAME for function names")
 #endif
 #ifndef DEBUG_BLTIN
-#warning "profiler needs DEBUG_BLTIN for builtin function names"
+#pragma message("WARNING: profiler needs DEBUG_BLTIN for builtin function names")
 #endif
 
 profiler prof;
@@ -342,8 +342,9 @@ void stack::runWithOrWithoutClosure(lambda *l, vars_t vars, vars_t parent)
   string& fileName=P.fileName;
   unsigned int offset=P.xmapCount;
 
-  bool traceless=!settings::getSetting<bool>("debug");
-  bool xasy=settings::getSetting<bool>("xasy") || offset;
+  bool traceless=!settings::debug;
+  bool xasy=settings::xasy || offset;
+
   if(xasy && curPos.filename() == fileName)
     topPos=curPos.shift(offset);
 
@@ -567,9 +568,12 @@ void stack::runWithOrWithoutClosure(lambda *l, vars_t vars, vars_t parent)
 #undef FRAMEVAR
 }
 
-void stack::load(string index) {
+void stack::loadModule(string index, Int numPushedParents) {
   vmFrame *inst=instMap[index];
   if (inst) {
+    for (Int i = 0; i < numPushedParents; ++i) {
+      pop();
+    }
     push(inst);
   }
   else {
